@@ -30,7 +30,7 @@ chxj_apply_convrule(request_rec *r, apr_array_header_t *convrules)
   for (ii = 0; ii < convrules->nelts; ii++) {
     pp = &entries[ii];
 
-    if (r->main != NULL) continue;
+    if (r->main) continue;
 
     /* Match */
     if (s_apply_rule(r, pp)) 
