@@ -24,6 +24,7 @@
 #include "chxj_buffered_write.h"
 #include "chxj_str_util.h"
 #include "chxj_header_inf.h"
+#include "chxj_conv_z2h.h"
 
 #define GET_CHTML10(X) ((chtml10_t *)(X))
 #undef W_L
@@ -289,6 +290,11 @@ tag_handler chtml10_handler[] = {
     s_chtml10_chxjif_tag,
     NULL,
   },
+  /* tagCHXJRAW */
+  {
+    s_chtml10_chxjif_tag,
+    NULL,
+  },
   /* tagNOBR */
   {
     NULL,
@@ -585,7 +591,7 @@ chxj_chtml10_emoji_only_converter(request_rec *r, device_table *spec, const char
   chtml10 = &__chtml10;
   doc     = &__doc;
 
-  DBG(r, "REQ[%X] start chxj_chtml10_emoji_only_converter()", (apr_size_t)(unsigned int)r);
+  DBG(r, "REQ[%X] start chxj_chtml10_emoji_only_converter()", (unsigned int)(apr_size_t)r);
   memset(doc,     0, sizeof(Doc));
   memset(chtml10, 0, sizeof(chtml10_t));
 
@@ -627,7 +633,7 @@ chxj_chtml10_emoji_only_converter(request_rec *r, device_table *spec, const char
 
   chtml10->out = chxj_buffered_write_flush(chtml10->out, &doc->buf);
 
-  DBG(r, "REQ[%X] end chxj_chtml10_emoji_only_converter()", (apr_size_t)(unsigned int)r);
+  DBG(r, "REQ[%X] end chxj_chtml10_emoji_only_converter()", (unsigned int)(apr_size_t)r);
   return chtml10->out;
 }
 
@@ -2052,6 +2058,11 @@ s_chtml10_start_input_tag(void *pdoc, Node *node)
   }
 
   if (value && *value != 0) {
+    if (type && (STRCASEEQ('s','S',"submit",type) || STRCASEEQ('r','R',"reset",type))) {
+      apr_size_t value_len = strlen(value);
+      value = chxj_conv_z2h(r, value, &value_len, chtml10->entryp);
+    }
+
     W_L(" value=\"");
     W_V(chxj_add_slash_to_doublequote(doc->pool, value));
     W_L("\"");
@@ -2993,6 +3004,7 @@ s_chtml10_text(void *pdoc, Node *child)
   chtml10_t   *chtml10;
   Doc         *doc;
   request_rec *r;
+  apr_size_t  z2h_input_len;
 
   chtml10 = GET_CHTML10(pdoc);
   doc     = chtml10->doc;
@@ -3045,6 +3057,9 @@ s_chtml10_text(void *pdoc, Node *child)
     }
   }
 
+  z2h_input_len = strlen(tdst);
+  tdst = chxj_conv_z2h(r, tdst, &z2h_input_len, chtml10->entryp);
+
   W_V(tdst);
   return chtml10->out;
 }
