@@ -197,14 +197,7 @@ chxj_headers_fixup(request_rec *r)
     DBG(r, "REQ[%X] end chxj_headers_fixup()", (unsigned int)(apr_size_t)r);
     return DECLINED;
   }
-  if (r->method_number == M_POST) {
-    if (!apr_table_get(r->headers_in, "X-Chxj-Forward")) {
-      s_clear_cookie_header(r, spec);
-    }
-  }
-  else {
-    s_clear_cookie_header(r, spec);
-  }
+
 
   switch(spec->html_spec_type) {
   case CHXJ_SPEC_Chtml_1_0:
@@ -240,6 +233,19 @@ chxj_headers_fixup(request_rec *r)
       apr_table_setn(r->headers_in, 
                      HTTP_USER_AGENT, 
                      entryp->user_agent);
+    /*
+     * this clear process must do before chxj_convert_input_header function.
+     */
+    if (entryp->action & CONVRULE_COOKIE_ON_BIT) {
+      if (r->method_number == M_POST) {
+        if (! apr_table_get(r->headers_in, "X-Chxj-Forward")) {
+          s_clear_cookie_header(r, spec);
+        }
+      }
+      else {
+        s_clear_cookie_header(r, spec);
+      }
+    }
 
     chxj_convert_input_header(r,entryp, spec);
 
@@ -307,6 +313,7 @@ chxj_headers_fixup(request_rec *r)
 static void
 s_clear_cookie_header(request_rec *r, device_table *spec)
 {
+  DBG(r, "REQ[%X] start s_clear_cookie_header()", TO_ADDR(r));
   switch(spec->html_spec_type) {
   case CHXJ_SPEC_Chtml_1_0:
   case CHXJ_SPEC_Chtml_2_0:
@@ -322,6 +329,7 @@ s_clear_cookie_header(request_rec *r, device_table *spec)
   default:
     break;
   }
+  DBG(r, "REQ[%X] end   s_clear_cookie_header()", TO_ADDR(r));
 }
 
 
@@ -1009,6 +1017,42 @@ chxj_output_filter(ap_filter_t *f, apr_bucket_brigade *bb)
   spec   = ctx->spec;
   dconf  = chxj_get_module_config(r->per_dir_config, &chxj_module);
 
+
+  if (apr_table_get(r->headers_out, "Location") || apr_table_get(r->err_headers_out, "Location")) {
+    DBG(r, "REQ[%X] found Location header", TO_ADDR(r));
+    if (entryp->action & CONVRULE_COOKIE_ON_BIT) {
+      cookie_lock_t *lock = NULL;
+      DBG(r, "REQ[%X] entryp->action == COOKIE_ON_BIT", TO_ADDR(r));
+      switch(spec->html_spec_type) {
+      case CHXJ_SPEC_Chtml_1_0:
+      case CHXJ_SPEC_Chtml_2_0:
+      case CHXJ_SPEC_Chtml_3_0:
+      case CHXJ_SPEC_Chtml_4_0:
+      case CHXJ_SPEC_Chtml_5_0:
+      case CHXJ_SPEC_Chtml_6_0:
+      case CHXJ_SPEC_Chtml_7_0:
+      case CHXJ_SPEC_XHtml_Mobile_1_0:
+      case CHXJ_SPEC_Jhtml:
+        lock = chxj_cookie_lock(r);
+        cookie = chxj_save_cookie(r);
+        s_add_cookie_id_if_has_location_header(r, cookie);
+        chxj_cookie_unlock(r, lock);
+        break;
+      default:
+        break;
+      }
+    }
+    if (! ap_is_HTTP_REDIRECT(r->status)) {
+      r->status = HTTP_MOVED_TEMPORARILY;
+    }
+    s_add_no_cache_headers(r, entryp);
+    /* must not send body. */
+    rv = pass_data_to_filter(f, "", 0);
+    DBG(f->r, "REQ[%X] end of chxj_output_filter()", TO_ADDR(r));
+    return rv;
+  }
+
+
   if (r->content_type) {
     if (! STRNCASEEQ('t','T',"text/html",r->content_type, sizeof("text/html")-1)
     &&  ! STRNCASEEQ('t','T',"text/xml", r->content_type, sizeof("text/xml")-1)
@@ -1704,6 +1748,7 @@ chxj_create_per_dir_config(apr_pool_t *p, char *arg)
   conf->cookie_timeout   = 0;
   conf->cookie_store_type = COOKIE_STORE_TYPE_NONE;
   conf->cookie_lazy_mode  = 0;
+  conf->cookie_dbm_type  = NULL;
 #if defined(USE_MYSQL_COOKIE)
   memset((void *)&conf->mysql, 0, sizeof(mysql_t));
   conf->mysql.port       = MYSQL_PORT;
@@ -1764,6 +1809,7 @@ chxj_merge_per_dir_config(apr_pool_t *p, void *basev, void *addv)
   mrg->forward_server_ip = NULL;
   mrg->allowed_cookie_domain = NULL;
   mrg->post_log         = NULL;
+  mrg->cookie_dbm_type  = NULL;
 
   mrg->dir = apr_pstrdup(p, add->dir);
 
@@ -1995,6 +2041,12 @@ chxj_merge_per_dir_config(apr_pool_t *p, void *basev, void *addv)
   else {
     mrg->post_log = base->post_log;
   }
+  if (add->cookie_dbm_type) {
+    mrg->cookie_dbm_type = add->cookie_dbm_type;
+  }
+  else {
+    mrg->cookie_dbm_type = base->cookie_dbm_type;
+  }
   return mrg;
 }
 
@@ -2866,6 +2918,23 @@ cmd_post_log_env(
   return NULL;
 }
 
+static const char *
+cmd_cookie_dbm_type(
+  cmd_parms   *cmd, 
+  void        *mconfig, 
+  const char  *arg)
+{
+  mod_chxj_config  *dconf;
+  if (strlen(arg) > 255)
+    return "mod_chxj: ChxjCookieDbmType is too long.";
+
+  dconf = (mod_chxj_config *)mconfig;
+
+  dconf->cookie_dbm_type = apr_pstrdup(cmd->pool, arg);
+
+  return NULL;
+}
+
 
 static const command_rec cmds[] = {
   AP_INIT_TAKE1(
@@ -3028,6 +3097,12 @@ static const command_rec cmds[] = {
     NULL,
     OR_ALL,
     "for CustomLog directive. mod_chxj's internal POST log environment name.(Default:chxj-post-log)"),
+  AP_INIT_TAKE1(
+    "ChxjCookieDbmType",
+    cmd_cookie_dbm_type,
+    NULL,
+    OR_ALL,
+    "Kind of DBM used with Cookie simulator.(default|GDBM|SDBM|DB|NDBM)"),
   {NULL,{NULL},NULL,0,0,NULL},
 };
 
